// Copyright 2021 Datafuse Labs.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

use std::any::Any;

use common_arrow::bitmap::MutableBitmap;
use common_exception::Result;

use crate::prelude::*;

pub trait MutableColumn {
    fn data_type(&self) -> DataTypeImpl;
    fn len(&self) -> usize;
    fn is_empty(&self) -> bool {
        self.len() == 0
    }
    fn as_any(&self) -> &dyn Any;
    fn as_mut_any(&mut self) -> &mut dyn Any;
    fn to_column(&mut self) -> ColumnRef;

    fn append_default(&mut self);
    fn append_data_value(&mut self, value: DataValue) -> Result<()>;
    fn pop_data_value(&mut self) -> Result<DataValue>;

    fn validity(&self) -> Option<&MutableBitmap> {
        None
    }

    fn shrink_to_fit(&mut self);
}
